from django.shortcuts import render
from django.contrib.auth.decorators import login_required,user_passes_test
from django.utils.decorators import method_decorator
from django.shortcuts import render
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status
import operator
from functools import reduce
from django.db.models import Q
from datetime import datetime
from django.db.models.functions import Coalesce
#from collections import Counter
from django.db.models import Count,Max,Sum
#import json
from django.utils import timezone
from .serializers import *
from clmainApp.utils import getBranchId,getStartEndDateTime,GetDaysBetweenDates
from clmainApp.models import Branch,WeekDay,FoundWay
from brvisitApp.models import PatientVisit
from brpatientApp.models import Patient,MARKETING_CODE_CHOICES,PatientInfo
from brfinanceApp.models import *
from brfinanceApp.serializers import *
from brpatientApp.serializers import PatientInfoMiniSerializer
from brmainApp.models import DrWorkingDay
from clfinanceApp.models import ExpenseCategory 

# Create your views here.
class PtSsnsPeriodByDoctorApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_PeriodByDoctorApi'), login_url='/auth/login/'))
    def get(self,request):
        branches = Branch.objects.filter(is_active=True).values('id','name')
        all_doctors = User.objects.filter(user_category="2DR")
        for branch in branches:
            branch["doctors"] = all_doctors.filter(groups__branch_id=branch["id"])
        #return Response(branches, status=status.HTTP_200_OK)
        return render(request,"Clinic/brreportApp/pt_ssns_period_by_dr.html",{"branches":branches,
        'activePage':'PtSsnsPeriodByDoctor'})
    def post(self,request):
        data=request.data
        data._mutable=True
        CurrentPeriodstartDateTime,CurrentPeriodendDateTime = getStartEndDateTime(data["startDate"],data["endDate"])
        lastPeriodstartDateTime,lastPeriodendDateTime = getStartEndDateTime(data["last_period_date_from"],data["last_period_date_to"])
        doctor_id = data["doctor_id"]
        branch_id = data["branch_id"]
        q_list = [Q(session_day__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]),Q(session_day__range=[lastPeriodstartDateTime,lastPeriodendDateTime])]
        psi_by_doctor_current_and_last = PatientSessionItem.objects.filter(reduce(operator.or_, q_list),session__patient__branch_id=branch_id,doctor_id=doctor_id).values("id","session__patient_id","session_day","session_status__name","session_status__color_code",)
        #psi_by_doctor_all = PatientSessionItem.objects.filter(session__patient__branch_id=branch_id,doctor_id=doctor_id,session_status_id="2AT").values("id","session__patient_id","session_day")
        psi_by_doctor_current = psi_by_doctor_current_and_last.filter(session_day__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime])
        psi_by_doctor_last = psi_by_doctor_current_and_last.filter(session_day__range=[lastPeriodstartDateTime,lastPeriodendDateTime])
        # Get unique patient IDs from psi_by_doctor_all queryset
        #unique_patient_ids = psi_by_doctor_all.distinct("session__patient_id").values_list("session__patient_id", flat=True)
        patient_ids = psi_by_doctor_current_and_last.values_list('session__patient_id',flat=True)
        visits_data = PatientVisit.objects.filter(patient_id__in=patient_ids).values('patient_id','visit_day')
        patient_total_session_items = psi_by_doctor_current_and_last.values('session__patient_id').annotate(psi_count=Count('id'))
        #patient_current_session_items = psi_by_doctor_current.values('session__patient_id').annotate(psi_count=Count('id'))
        # Extract the unique patient IDs from the queryset
        unique_patient_ids_list = [patient['session__patient_id'] for patient in patient_total_session_items]
        # Convert the queryset result to a list if needed
        #unique_patient_ids_list = list(unique_patient_ids)
        patients_info = Patient.objects.filter(id__in = unique_patient_ids_list).values("id","fullname","code","dr_notes","rcp_notes")
        response_data = {}
        patients_data = []
        days_list = GetDaysBetweenDates(data["startDate"],data["endDate"])
        
        weekdays = WeekDay.objects.filter(is_active=True)
        doctor_working_days_ids = DrWorkingDay.objects.filter(dr_id=doctor_id,branch_id=branch_id).values_list('weekday_id',flat=True)
        days_list_header = []
        date_format = "%Y-%m-%d"
        for day in days_list :
            weekday_objs = weekdays.filter(code=day.weekday())
            if weekday_objs :
                weekday_obj = weekday_objs[0]
                if weekday_obj.id in doctor_working_days_ids :
                    days_list_header.append({"day_name":weekday_obj.name,"day_date":day,"day_str":day.strftime(date_format)})
        response_data["days_list_header"] = days_list_header
        doctor_days_list = [day["day_date"] for day in days_list_header]
        for pt in patients_info :
            # patient name , last visit data , last sessions count assigned , last sessions count attend , 
            # current visits count assigned , current sessions count attend , loop over days with status and colors 
            # Create a new dictionary with the same keys and values as patient_info
            patient = dict(pt) 
            max_visit_day = visits_data.filter(patient_id=pt["id"]).aggregate(Max('visit_day'))['visit_day__max']
            patient["last_visit_date"] = max_visit_day
            patient_psi_last = psi_by_doctor_last.filter(session__patient_id=pt["id"])
            patient_psi_last_count_all = (patient_psi_last.values('id').aggregate(count_last_sessions=Coalesce(Count('id'),0)))['count_last_sessions']
            patient["psi_last_count_all"] = patient_psi_last_count_all
            patient_psi_last_count_attend = (patient_psi_last.filter(session_status_id="2AT").values('id').aggregate(count_last_sessions=Coalesce(Count('id'),0)))['count_last_sessions']
            patient["psi_last_count_attend"] = patient_psi_last_count_attend
            patient_psi_current = psi_by_doctor_current.filter(session__patient_id=pt["id"])
            patient_psi_current_count_all = (patient_psi_current.values('id').aggregate(count_current_sessions=Coalesce(Count('id'),0)))['count_current_sessions']
            patient["psi_current_count_all"] = patient_psi_current_count_all
            patient_psi_current_count_attend = (patient_psi_current.filter(session_status_id="2AT").values('id').aggregate(count_current_sessions=Coalesce(Count('id'),0)))['count_current_sessions']
            patient["psi_current_count_attend"] = patient_psi_current_count_attend
            pt_sessions_days_data =[]
            for day in doctor_days_list :
                day_objs = patient_psi_current.filter(session_day=day)
                if day_objs : 
                    pt_sessions_days_data.append({"status_name":day_objs[0]["session_status__name"],
                                                  "status_color":day_objs[0]["session_status__color_code"]})
                else :
                    pt_sessions_days_data.append(None)
            patient["sessions_days_data"]=pt_sessions_days_data
            #patient_dict = {**pt}
            ## Count objects for each session__patient_id and annotate the count as 'object_count'
            #patient_object_counts = psi_by_doctor_all.values('session__patient_id').annotate(object_count=Count('id'))
            # current_psi_data = patient_current_session_items.filter(session__patient_id = pt["id"])
            # current_psi_count = current_psi_data[0]["psi_count"] if current_psi_data else 0
            # patient["current_psi_count"] = current_psi_count
            # total_psi_data = patient_total_session_items.filter(session__patient_id = pt["id"])
            # total_psi_count = total_psi_data[0]["psi_count"] if total_psi_data else 0
            # patient["old_psi_count"] = total_psi_count - current_psi_count
            patients_data.append(patient)
        response_data["patients_data"] = patients_data
        return Response(response_data,status=status.HTTP_200_OK)
    
class PtSsnsGeneralByDoctorApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_GeneralByDoctorApi'), login_url='/auth/login/'))
    def get(self,request):
        branches = Branch.objects.filter(is_active=True).values('id','name')
        all_doctors = User.objects.filter(user_category="2DR")
        for branch in branches:
            branch["doctors"] = all_doctors.filter(groups__branch_id=branch["id"])
        return render(request,"Clinic/brreportApp/pt_ssns_general_by_dr.html",{"branches":branches,
        'activePage':'PtSsnsGeneralByDoctor'})
    def post(self,request):
        data=request.data
        doctor_id = data["doctor_id"]
        branch_id = data["branch_id"]
        psi_by_doctor_all = PatientSessionItem.objects.filter(session__patient__branch_id=branch_id,doctor_id=doctor_id,session_status_id="2AT").values("id","session__patient_id","session_day")
        patient_total_session_items = psi_by_doctor_all.values('session__patient_id').annotate(psi_count=Count('id'))
        unique_patient_ids_list = [patient['session__patient_id'] for patient in patient_total_session_items]
        patients_info = Patient.objects.filter(id__in = unique_patient_ids_list).values("id","fullname","code","dr_notes","rcp_notes")
        response_data = []
        for pt in patients_info :
            patient = dict(pt)
            patient["last_session_day"]= psi_by_doctor_all.filter(session__patient_id=pt["id"]).aggregate(max_session_day=Max('session_day'))['max_session_day']
            total_psi_data = patient_total_session_items.filter(session__patient_id = pt["id"])
            patient["total_psi_count"] = total_psi_data[0]["psi_count"] if total_psi_data else 0
            response_data.append(patient)
        return Response(response_data,status=status.HTTP_200_OK)

class MarketingDataApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_AdsPatientData'), login_url='/auth/login/'))
    def get(self,request):
        branches = Branch.objects.filter(is_active=True).values('id','name')
        return render(request,"Clinic/brreportApp/marketing_data.html",{"branches":branches,'MARKETING_CODE_CHOICES':MARKETING_CODE_CHOICES,
                        'activePage':'MarketingData'})
    def post(self,request):
        data=request.data
        #import pdb
        #pdb.set_trace()
        #marketing_code = data["marketing_code"]# 1 attend 0 doesnt attend before
        marketing_codes = data.getlist('marketing_codes[]') 
        branch_id = data["branch_id"]
        startDateTime,endDateTime = getStartEndDateTime(data["startDate"],data["endDate"])
        if branch_id != "0": patients  = PatientInfo.objects.filter(marketing_code__in=marketing_codes,patient__branch_id=branch_id,patient__created_at__range=[startDateTime,endDateTime])
        else : patients  = PatientInfo.objects.filter(marketing_code__in=marketing_codes,patient__created_at__range=[startDateTime,endDateTime])
        patient_ids = patients.values_list("patient_id",flat=True)
        psi_items = PatientSessionItem.objects.filter(session__patient_id__in =patient_ids ,session_status_id="2AT").values('session__patient_id').annotate(count_pt_sessions=Coalesce(Count('id'),0))
        serializer = PatientInfoMiniSerializer(patients,many=True)
        serializer_data = serializer.data
        for pt_obj in serializer_data :
            pt_sessions_obj =  psi_items.filter(session__patient_id=pt_obj['patient'])
            pt_obj['sessions'] = pt_sessions_obj[0]["count_pt_sessions"] if pt_sessions_obj else 0
        
        return Response(serializer_data,status=status.HTTP_200_OK)

class AdsPatientDataApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_AdsPatientData'), login_url='/auth/login/'))
    def get(self,request):
        branches = Branch.objects.filter(is_active=True).values('id','name')
        return render(request,"Clinic/brreportApp/ads_patient_data.html",{"branches":branches,'activePage':'AdsPatientData'})
    
    def post(self,request):
        data=request.data
        marketing_code = data["marketing_code"]# 1 attend 0 doesnt attend before 
        #visit_type = data["visit_type"] # 0 all - 1 visit - 2 session
        branch_id = data["branch_id"]
        #q_visits_list = [] 
        #q_sessions_list = [] 
        if branch_id != "0":
            if visit_type == "0":# 0 all - 1 visit - 2 session - 3 visit no session 
                patients_with_sessions = PatientSessionItem.objects.filter(session__patient__branch_id=branch_id,session_status_id="2AT").values('session__patient_id').annotate(max_session_day=Max('session_day'))
                patients_ids__with_sessions = patients_with_sessions.values_list('session__patient_id',flat=True)
                patients_with_visits = PatientVisit.objects.filter(visit_branch_id=branch_id).values(
                    'patient_id').annotate(max_visit_day=Max('visit_day'))
                    #'patient_id', 'patient__fullname','patient__number_1','patient__number_2').annotate(max_visit_day=Max('visit_day'))
                patients_ids__with_visits = patients_with_visits.values_list('patient_id',flat=True) 
            elif visit_type == "1" : # 0 all - 1 visit - 2 session
                patients_ids__with_sessions=[]
                patients_with_visits = PatientVisit.objects.filter(visit_branch_id=branch_id).values('patient_id').annotate(max_visit_day=Max('visit_day'))
                patients_ids__with_visits = patients_with_visits.values_list('patient_id',flat=True) 
            elif visit_type == "2" : # 0 all - 1 visit - 2 session
                patients_with_sessions = PatientSessionItem.objects.filter(session__patient__branch_id=branch_id,session_status_id="2AT").values('session__patient_id').annotate(max_session_day=Max('session_day'))
                patients_ids__with_sessions = patients_with_sessions.values_list('session__patient_id',flat=True)                
                patients_ids__with_visits=[]
            elif visit_type == "3":# 0 all - 1 visit - 2 session - 3 visit no session 
                patients_with_sessions = PatientSessionItem.objects.filter(session__patient__branch_id=branch_id,session_status_id="2AT").values('session__patient_id').annotate(max_session_day=Max('session_day'))
                patients_ids__with_sessions_list = patients_with_sessions.values_list('session__patient_id',flat=True)
                patients_with_visits = PatientVisit.objects.filter(visit_branch_id=branch_id).values(
                    'patient_id').annotate(max_visit_day=Max('visit_day'))
                    #'patient_id', 'patient__fullname','patient__number_1','patient__number_2').annotate(max_visit_day=Max('visit_day'))
                patients_ids__with_visits_list = patients_with_visits.values_list('patient_id',flat=True)
                #get patients in visits but not in sessions 
                # Convert lists to sets
                visits_set = set(patients_ids__with_visits_list)
                sessions_set = set(patients_ids__with_sessions_list)
                # Find elements in visits_set that are not in sessions_set
                elements_not_in_sessions = visits_set - sessions_set
                # Convert the result back to a list if needed
                pt_ids_visits_not_in_sessions = list(elements_not_in_sessions)
                patients_ids__with_sessions=[]
                patients_ids__with_visits=pt_ids_visits_not_in_sessions
            else : 
                patients_ids__with_visits=[]
                patients_ids__with_sessions=[]
        else :
            if visit_type == "0":# 0 all - 1 visit - 2 session
                patients_with_sessions = PatientSessionItem.objects.filter(session_status_id="2AT").values('session__patient_id').annotate(max_session_day=Max('session_day'))
                patients_ids__with_sessions = patients_with_sessions.values_list('session__patient_id',flat=True)
                patients_with_visits = PatientVisit.objects.values('patient_id').annotate(max_visit_day=Max('visit_day'))
                    #'patient_id', 'patient__fullname','patient__number_1','patient__number_2').annotate(max_visit_day=Max('visit_day'))
                patients_ids__with_visits = patients_with_visits.values_list('patient_id',flat=True) 
            elif visit_type == "1" : # 0 all - 1 visit - 2 session
                patients_ids__with_sessions=[]
                patients_with_visits = PatientVisit.objects.values('patient_id').annotate(max_visit_day=Max('visit_day'))
                patients_ids__with_visits = patients_with_visits.values_list('patient_id',flat=True) 
            elif visit_type == "2" : # 0 all - 1 visit - 2 session
                patients_with_sessions = PatientSessionItem.objects.filter(session_status_id="2AT").values('session__patient_id').annotate(max_session_day=Max('session_day'))
                patients_ids__with_sessions = patients_with_sessions.values_list('session__patient_id',flat=True)                
                patients_ids__with_visits=[]
            elif visit_type == "3":# 0 all - 1 visit - 2 session - 3 visit no session 
                patients_with_sessions = PatientSessionItem.objects.values('session__patient_id').annotate(max_session_day=Max('session_day'))
                patients_ids__with_sessions_list = patients_with_sessions.values_list('session__patient_id',flat=True)
                patients_with_visits = PatientVisit.objects.values(
                    'patient_id').annotate(max_visit_day=Max('visit_day'))
                    #'patient_id', 'patient__fullname','patient__number_1','patient__number_2').annotate(max_visit_day=Max('visit_day'))
                patients_ids__with_visits_list = patients_with_visits.values_list('patient_id',flat=True)
                #get patients in visits but not in sessions 
                # Convert lists to sets
                visits_set = set(patients_ids__with_visits_list)
                sessions_set = set(patients_ids__with_sessions_list)
                # Find elements in visits_set that are not in sessions_set
                elements_not_in_sessions = visits_set - sessions_set
                # Convert the result back to a list if needed
                pt_ids_visits_not_in_sessions = list(elements_not_in_sessions)
                patients_ids__with_sessions=[]
                patients_ids__with_visits=pt_ids_visits_not_in_sessions
            else : 
                patients_ids__with_visits=[]
                patients_ids__with_sessions=[]
        patients_ids_with_data_all = list(patients_ids__with_sessions) + list(patients_ids__with_visits)
        patients_data = []
        if is_attended == "0" : 
            if branch_id != "0":
                patients_without_visits_sessions = Patient.objects.filter(branch_id=branch_id).exclude(pk__in=patients_ids_with_data_all).values("id","fullname","number_1","number_2")
            else :
                patients_without_visits_sessions = Patient.objects.exclude(pk__in=patients_ids_with_data_all).values("id","fullname","number_1","number_2")

            for pt_obj in patients_without_visits_sessions:
                pt_id = pt_obj["id"]
                patient_obj ={"id":pt_id,"fullname":pt_obj["fullname"],"number_1":pt_obj["number_1"],"number_2":pt_obj["number_2"]}
                patient_obj["last_session_day"] =  (patients_with_sessions.filter(session__patient_id=pt_id).first())["max_session_day"]  if pt_id in patients_ids__with_sessions else None
                patient_obj["last_visit_day"] =  (patients_with_visits.filter(patient_id=pt_id).first())["max_visit_day"]  if pt_id in patients_ids__with_visits else None
                patients_data.append(patient_obj)
        else :
            patients_with_visits_sessions = Patient.objects.filter(pk__in=patients_ids_with_data_all).values("id","fullname","number_1","number_2")
            for pt_obj in patients_with_visits_sessions:
                pt_id = pt_obj["id"]
                patient_obj ={"id":pt_id,"fullname":pt_obj["fullname"],"number_1":pt_obj["number_1"],"number_2":pt_obj["number_2"]}
                patient_obj["last_session_day"] =  (patients_with_sessions.filter(session__patient_id=pt_id).first())["max_session_day"]  if pt_id in patients_ids__with_sessions else None
                patient_obj["last_visit_day"] =  (patients_with_visits.filter(patient_id=pt_id).first())["max_visit_day"]  if pt_id in patients_ids__with_visits else None
                patients_data.append(patient_obj)
        return Response(patients_data,status=status.HTTP_200_OK)

class AdsPatientPeriodDataApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_AdsPatientData'), login_url='/auth/login/'))
    def get(self,request):
        branches = Branch.objects.filter(is_active=True).values('id','name')
        return render(request,"Clinic/brreportApp/ads_patient_data_period.html",{"branches":branches,'activePage':'AdsPatientPeriodData'})
    
    def post(self,request):
        data=request.data
        is_attended = data["is_attended"]# 1 attend 0 doesnt attend before 
        visit_type = data["visit_type"] # 0 all - 1 visit - 2 session
        branch_id = data["branch_id"]
        CurrentPeriodstartDateTime,CurrentPeriodendDateTime = getStartEndDateTime(data["startDate"],data["endDate"])
        #q_list = [Q(created_at__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime])]
        #q_visits_list = [] 
        #q_sessions_list = [] 
        if branch_id != "0":
            if visit_type == "0":# 0 all - 1 visit - 2 session - 3 visit no session 
                patients_with_sessions = PatientSessionItem.objects.filter(session__patient__branch_id=branch_id,session_status_id="2AT",session_day__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]).values('session__patient_id').annotate(max_session_day=Max('session_day'))
                patients_ids__with_sessions = patients_with_sessions.values_list('session__patient_id',flat=True)
                patients_with_visits = PatientVisit.objects.filter(visit_branch_id=branch_id,created_at__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]).values(
                    'patient_id').annotate(max_visit_day=Max('visit_day'))
                    #'patient_id', 'patient__fullname','patient__number_1','patient__number_2').annotate(max_visit_day=Max('visit_day'))
                patients_ids__with_visits = patients_with_visits.values_list('patient_id',flat=True) 
            elif visit_type == "1" : # 0 all - 1 visit - 2 session
                patients_ids__with_sessions=[]
                patients_with_visits = PatientVisit.objects.filter(visit_branch_id=branch_id,created_at__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]).values('patient_id').annotate(max_visit_day=Max('visit_day'))
                patients_ids__with_visits = patients_with_visits.values_list('patient_id',flat=True) 
            elif visit_type == "2" : # 0 all - 1 visit - 2 session
                patients_with_sessions = PatientSessionItem.objects.filter(session__patient__branch_id=branch_id,session_status_id="2AT",session_day__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]).values('session__patient_id').annotate(max_session_day=Max('session_day'))
                patients_ids__with_sessions = patients_with_sessions.values_list('session__patient_id',flat=True)                
                patients_ids__with_visits=[]
            elif visit_type == "3":# 0 all - 1 visit - 2 session - 3 visit no session 
                patients_with_sessions = PatientSessionItem.objects.filter(session__patient__branch_id=branch_id,session_status_id="2AT",session_day__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]).values('session__patient_id').annotate(max_session_day=Max('session_day'))
                patients_ids__with_sessions_list = patients_with_sessions.values_list('session__patient_id',flat=True)
                patients_with_visits = PatientVisit.objects.filter(visit_branch_id=branch_id,created_at__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]).values(
                    'patient_id').annotate(max_visit_day=Max('visit_day'))
                    #'patient_id', 'patient__fullname','patient__number_1','patient__number_2').annotate(max_visit_day=Max('visit_day'))
                patients_ids__with_visits_list = patients_with_visits.values_list('patient_id',flat=True)
                #get patients in visits but not in sessions 
                # Convert lists to sets
                visits_set = set(patients_ids__with_visits_list)
                sessions_set = set(patients_ids__with_sessions_list)
                # Find elements in visits_set that are not in sessions_set
                elements_not_in_sessions = visits_set - sessions_set
                # Convert the result back to a list if needed
                pt_ids_visits_not_in_sessions = list(elements_not_in_sessions)
                patients_ids__with_sessions=[]
                patients_ids__with_visits=pt_ids_visits_not_in_sessions
            else : 
                patients_ids__with_visits=[]
                patients_ids__with_sessions=[]
        else :#branch data 
            if visit_type == "0":# 0 all - 1 visit - 2 session
                patients_with_sessions = PatientSessionItem.objects.filter(session_status_id="2AT",session_day__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]).values('session__patient_id').annotate(max_session_day=Max('session_day'))
                patients_ids__with_sessions = patients_with_sessions.values_list('session__patient_id',flat=True)
                patients_with_visits = PatientVisit.objects.filter(created_at__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]).values('patient_id').annotate(max_visit_day=Max('visit_day'))
                    #'patient_id', 'patient__fullname','patient__number_1','patient__number_2').annotate(max_visit_day=Max('visit_day'))
                patients_ids__with_visits = patients_with_visits.values_list('patient_id',flat=True) 
            elif visit_type == "1" : # 0 all - 1 visit - 2 session
                patients_ids__with_sessions=[]
                patients_with_visits = PatientVisit.objects.filter(created_at__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]).values('patient_id').annotate(max_visit_day=Max('visit_day'))
                patients_ids__with_visits = patients_with_visits.values_list('patient_id',flat=True) 
            elif visit_type == "2" : # 0 all - 1 visit - 2 session
                patients_with_sessions = PatientSessionItem.objects.filter(session_status_id="2AT",session_day__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]).values('session__patient_id').annotate(max_session_day=Max('session_day'))
                patients_ids__with_sessions = patients_with_sessions.values_list('session__patient_id',flat=True)                
                patients_ids__with_visits=[]
            elif visit_type == "3":# 0 all - 1 visit - 2 session - 3 visit no session 
                patients_with_sessions = PatientSessionItem.objects.filter(session_day__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]).values('session__patient_id').annotate(max_session_day=Max('session_day'))
                patients_ids__with_sessions_list = patients_with_sessions.values_list('session__patient_id',flat=True)
                patients_with_visits = PatientVisit.objects.filter(created_at__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]).values(
                    'patient_id').annotate(max_visit_day=Max('visit_day'))
                    #'patient_id', 'patient__fullname','patient__number_1','patient__number_2').annotate(max_visit_day=Max('visit_day'))
                patients_ids__with_visits_list = patients_with_visits.values_list('patient_id',flat=True)
                #get patients in visits but not in sessions 
                # Convert lists to sets
                visits_set = set(patients_ids__with_visits_list)
                sessions_set = set(patients_ids__with_sessions_list)
                # Find elements in visits_set that are not in sessions_set
                elements_not_in_sessions = visits_set - sessions_set
                # Convert the result back to a list if needed
                pt_ids_visits_not_in_sessions = list(elements_not_in_sessions)
                patients_ids__with_sessions=[]
                patients_ids__with_visits=pt_ids_visits_not_in_sessions
            else : 
                patients_ids__with_visits=[]
                patients_ids__with_sessions=[]
        patients_ids_with_data_all = list(patients_ids__with_sessions) + list(patients_ids__with_visits)
        patients_data = []
        if is_attended == "0" : 
            if branch_id != "0":
                patients_without_visits_sessions = Patient.objects.filter(branch_id=branch_id).exclude(pk__in=patients_ids_with_data_all).values("id","fullname","number_1","number_2")
            else :
                patients_without_visits_sessions = Patient.objects.exclude(pk__in=patients_ids_with_data_all).values("id","fullname","number_1","number_2")

            for pt_obj in patients_without_visits_sessions:
                pt_id = pt_obj["id"]
                patient_obj ={"id":pt_id,"fullname":pt_obj["fullname"],"number_1":pt_obj["number_1"],"number_2":pt_obj["number_2"]}
                patient_obj["last_session_day"] =  (patients_with_sessions.filter(session__patient_id=pt_id).first())["max_session_day"]  if pt_id in patients_ids__with_sessions else None
                patient_obj["last_visit_day"] =  (patients_with_visits.filter(patient_id=pt_id).first())["max_visit_day"]  if pt_id in patients_ids__with_visits else None
                patients_data.append(patient_obj)
        else :
            patients_with_visits_sessions = Patient.objects.filter(pk__in=patients_ids_with_data_all).values("id","fullname","number_1","number_2")
            for pt_obj in patients_with_visits_sessions:
                pt_id = pt_obj["id"]
                patient_obj ={"id":pt_id,"fullname":pt_obj["fullname"],"number_1":pt_obj["number_1"],"number_2":pt_obj["number_2"]}
                patient_obj["last_session_day"] =  (patients_with_sessions.filter(session__patient_id=pt_id).first())["max_session_day"]  if pt_id in patients_ids__with_sessions else None
                patient_obj["last_visit_day"] =  (patients_with_visits.filter(patient_id=pt_id).first())["max_visit_day"]  if pt_id in patients_ids__with_visits else None
                patients_data.append(patient_obj)
        return Response(patients_data,status=status.HTTP_200_OK)

from django.db.models.functions import TruncDate

class FoundWaysDataApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_FoundWaysData'), login_url='/auth/login/'))
    def get(self,request):
        branches = Branch.objects.filter(is_active=True).values('id','name')
        foundways = FoundWay.objects.filter(is_active=True).values('id','name')
        return render(request,"Clinic/brreportApp/xxxxx.html",
                      {"branches":branches,"foundways":foundways,
                       'activePage':'FoundWaysData'})
    def post(self,request):
        data=request.data
        foundway_id = data["foundway_id"]
        branch_id = data["branch_id"]
        CurrentPeriodstartDateTime,CurrentPeriodendDateTime = getStartEndDateTime(data["startDate"],data["endDate"])
        q_list = [Q(created_at__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime])]
        if branch_id != "0":q_list.append(Q(branch_id=branch_id))
        if foundway_id != "0":q_list.append(Q(foundway_id=foundway_id))
        patients = Patient.objects.filter(reduce(operator.and_, q_list)).values("id","fullname","code","number_1","number_2","created_at","foundway__name")
        patients_ids = patients.values_list('id',flat=True)
        patients_psi = PatientSessionItem.objects.filter(session__patient_id__in=patients_ids).values('session__patient_id').annotate(max_session_day=Max('session_day'))
        patients_pv = PatientVisit.objects.filter(patient_id__in=patients_ids).values('patient_id').annotate(max_visit_day=Max('visit_day'))
        #import pdb
        #pdb.set_trace()
        patients_ids__with_sessions = [psi["session__patient_id"] for psi in patients_psi]
        #patients_ids__with_sessions=patients_psi.values("session__patient_id",flat=True)
        patients_ids__with_visits = [pv["patient_id"] for pv in patients_pv]
        #patients_ids__with_visits=patients_pv.values("patient_id",flat=True)
        response_data={}
        patients_data = []
        for pt_obj in patients:
                pt_id = pt_obj["id"]
                patient_obj ={"id":pt_id,"fullname":pt_obj["fullname"],"number_1":pt_obj["number_1"],"number_2":pt_obj["number_2"]}
                patient_obj["last_session_day"] =  (patients_psi.filter(session__patient_id=pt_id).first())["max_session_day"]  if pt_id in patients_ids__with_sessions else None
                patient_obj["last_visit_day"] =  (patients_pv.filter(patient_id=pt_id).first())["max_visit_day"]  if pt_id in patients_ids__with_visits else None
                patients_data.append(patient_obj)
        response_data["patients_data"]=patients_data
        date_format = "%Y-%m-%d"
        if foundway_id != "0":#days over patients count 
            # Annotate the queryset to get the count of patients for each day
            daily_patient_count = patients.annotate(creation_date=TruncDate('created_at')).values('creation_date').annotate(day_pt_count=Count('id')).order_by('creation_date')
            response_data['daily_patients_data'] = {"names": list(map(lambda item: item['creation_date'].strftime(date_format), daily_patient_count)),"values":list(map(lambda item: item['day_pt_count'], daily_patient_count))}
        else :
            pt_foundways_data = patients.values("foundway__name").annotate(pt_count = Count('id'))
            response_data['pt_foundways_data'] = {"names": list(map(lambda item: item['foundway__name'], pt_foundways_data)),"values":list(map(lambda item: item['pt_count'], pt_foundways_data))}
        return Response(response_data,status=status.HTTP_200_OK)

class PatientsDebtsApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_PatientsDebts'), login_url='/main/login/'))
    def get(self,request):
        branches = Branch.objects.filter(is_active=True).values('id','name')
        return render(request,"Clinic/brreportApp/patients_debts.html",{"branches":branches,'activePage':'PatientsDebts'})
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_PatientsDebts'), login_url='/main/login/'))    
    def post(self,request):
        data = request.data
        CurrentPeriodstartDateTime,CurrentPeriodendDateTime = getStartEndDateTime(data["startDate"],data["endDate"])
        #q_list = [Q(created_on__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]),Q(branch_id=branch_id)]
        min_debt= data["min_debt"]
        branch_id = data["branch_id"]
        p_sessions = PatientSession.objects.filter(patient__branch_id=branch_id,amount_remaining__gt=min_debt,is_debt_hide=False,created_at__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]).order_by('patient_id','created_at',)
        p_visits = PatientVisit.objects.filter(visit_branch_id=branch_id,amount_remaining__gt=min_debt,is_debt_hide=False,created_at__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]).order_by('patient_id','created_at',)
        #import pdb
        #pdb.set_trace()
        #invoices = PatientInvoice.objects.filter(amount_paid__lt = F('total_finished')).annotate(current_debt=F('total_finished')-F('amount_paid')).order_by('current_debt','patient').values('id','current_debt','total_finished','amount_paid','patient')
        #PatientInvoiceCustomSerializer
        p_sessions_srl= PatientSessionForDebtSerializer(p_sessions,many=True)
        p_visits_srl= PvForDebtSerializer(p_visits,many=True)
        data = {"p_sessions":p_sessions_srl.data,"p_visits":p_visits_srl.data}
        return Response(data,status=status.HTTP_200_OK)
        #else :return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

class PatientDebtApi(APIView):
    def get_object_pv(self,pk):
        try:
            return PatientVisit.objects.get(pk=pk)
        except PatientVisit.DoesNotExist:
            return None    
    
    def get_object_ps(self,pk):
        try:
            return PatientSession.objects.get(pk=pk)
        except PatientSession.DoesNotExist:
            return None
        
    #@method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.change_group'), login_url='/auth/login/'))
    def post(self,request,pk):
        data=request.data
        object_type = data["object_type"]
        #reponse_data = {}
        if object_type == "pv" : 
            obj=self.get_object_pv(pk)
            serializer= PvMiniForDebtSerializer(obj)
        elif object_type == "ps" : 
            obj=self.get_object_ps(pk)
            serializer= PsMiniForDebtSerializer(obj)
        else : serializer = {"data":None}
        return Response(serializer.data,status=status.HTTP_200_OK)

    #@method_decorator(user_passes_test(lambda u: u.has_perm('screen.view_PatientVisit'), login_url='/auth/login/'))        
    # def get(self,request,pk): 
    #     patient_visitObj=self.get_object(pk)
    #     if patient_visitObj :
    #         #return render(request,"mainApp/patient_visit.html",{"patient_visit":patient_visitObj,'activePage':'PatientVisits'})
    #         serializeobj= PatientVisitSerializer(patient_visitObj)
    #         return Response(serializeobj.data,status=status.HTTP_200_OK)
    #     else : return Response(status=status.HTTP_404_NOT_FOUND)
    
    #@method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_PatientsDebts'), login_url='/main/login/'))    
    def put(self,request,pk):
        data=request.data
        object_type = data["obj_type"]
        obj_id = data["obj_id"]
        contact_notes = data["contact_notes"]
        debt_id = data["debt_id"] if data["debt_id"] != "" else None
        user_id = request.user.id
        request.data.update({})
        data = { "notes":contact_notes,"updated_by_id":user_id,"updated_at":timezone.now(),}
        try :
            if object_type == "pv" : 
                data["pv_id"]= obj_id
                #pdb.set_trace()
                obj, created = PvDebt.objects.update_or_create(
                    id=debt_id,  # Lookup condition
                    defaults=data
                )
            elif object_type == "ps" :
                data["pt_ssn_id"]= obj_id 
                #pdb.set_trace()
                obj, created = PtSsnDebt.objects.update_or_create(
                    id=debt_id,  # Lookup condition
                    defaults=data
                )
            return Response(status=status.HTTP_200_OK)
        except Exception as err_msg: 
                return Response(data ="{}".format(str(err_msg)),status=status.HTTP_400_BAD_REQUEST)

    #@method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_PatientsDebts'), login_url='/main/login/'))    
    def delete(self,request,pk):
        data=request.data
        object_type = data["object_type"]
        try :
            if object_type == "pv" :obj=self.get_object_pv(pk)
            elif object_type == "ps" :obj=self.get_object_ps(pk)
            obj.is_debt_hide = True
            obj.save()
            return Response(status=status.HTTP_200_OK)
        except Exception as err_msg: 
                return Response(data ="{}".format(str(err_msg)),status=status.HTTP_400_BAD_REQUEST)
    

class IrregularPtDataReportApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_IrregularPtData'), login_url='/auth/login/'))
    def get(self,request):
        branches = Branch.objects.filter(id = getBranchId(request)).values('id','name')
        return render(request,"Branch/brreportApp/irregular_pt_data_branch.html",{"branches":branches,"page_source":"Branch", 'activePage':'IrregularPtData'})

'''
from twilio.rest import Client
account_sid = 'AC93b3c4c9eb4271d1fe2c3da1affc5a74'
auth_token = 'f8ea44f2e7b74fe82926dc1df4237ecc'
account_whatsapp_number = "+14155238886"
#http://vmi873706.contaboserver.net:7777/br_rpt/ads_patient_data/?phone={{phone}}
class TestWhatsappApiTwilio(APIView):
    #@method_decorator(user_passes_test(lambda u: u.has_perm('Management.view_expenseitem'), login_url='/auth/login/'))
    def post(self,request):
        data = request.data
        print(data)
        #received Obj
        profileName=data["ProfileName"]
        msgBody=data["Body"]
        smsStatus=data["SmsStatus"]#'received'
        senderNumberVal = data["From"]
        accountNumberVal = data["To"]
        senderNumber = senderNumberVal.split(":")[1]
        #<QueryDict: {'SmsMessageSid': ['SM31f468adfa886fdb1184c7e08b315e34'], 'NumMedia': ['0'], 'ProfileName': ['Muhammed'], 'SmsSid': ['SM31f468adfa886fdb1184c7e08b315e34'], 'WaId': ['201093211000'], 'SmsStatus': ['received'], 'Body': ['Hi'], 'To': ['whatsapp:+14155238886'], 'NumSegments': ['1'], 'ReferralNumMedia': ['0'], 'MessageSid': ['SM31f468adfa886fdb1184c7e08b315e34'], 'AccountSid': ['AC93b3c4c9eb4271d1fe2c3da1affc5a74'], 'From': ['whatsapp:+201093211000'], 'ApiVersion': ['2010-04-01']}>
        #reponse_data = {"type" : "whatsapp", "data" : [ {"id" : 2, "wid" : "+201018223004", "phone" : "+201155186508", "message" : "Hello World!"}]}
        client = Client(account_sid, auth_token)
        client.messages.create(
        from_=accountNumberVal,
        body=f'Thank You {profileName} For Contacting us',
        to=senderNumberVal
        )
        # client.messages.create(
        # from_='whatsapp:+14155238886',
        # body='Your Yummy Cupcakes Company order of 1 dozen frosted cupcakes has shipped and should be delivered on July 10, 2019. Details: http://www.yummycupcakes.com/',
        # to='whatsapp:+201093211000'
        # )
        # reponse_list = [{"type" : "whatsapp", # type of payload: received whatsapp chat
        #         "data" : [
        #         "id" : 2, # unique id from the system
        #         "wid" : "+639760713666", # whatsapp account phone number
        #         "phone" : "+639760666713", # sender phone number
        #         "message" : "Hello World!", # message
        #         #"attachment" => "http:#imageurl.com/image.jpg", # attachment url
        #         #"timestamp" => 1645684231 # receive timestamp
        #     ]},
        # ]
        return Response({},status=status.HTTP_200_OK)
    
    def get(self,request):
        print(request.data)
        reponse_data = {"type" : "whatsapp", "data" : [ {"id" : 2, "wid" : "+201018223004", "phone" : "+201155186508", "message" : "Hello World!"}]}
        # reponse_list = [{"type" : "whatsapp", # type of payload: received whatsapp chat
        #         "data" : [
        #         "id" : 2, # unique id from the system
        #         "wid" : "+639760713666", # whatsapp account phone number
        #         "phone" : "+639760666713", # sender phone number
        #         "message" : "Hello World!", # message
        #         #"attachment" => "http:#imageurl.com/image.jpg", # attachment url
        #         #"timestamp" => 1645684231 # receive timestamp
        #     ]},
        # ]
        return Response(reponse_data,status=status.HTTP_200_OK)

class TestWhatsappApiKingMaster(APIView):
    #@method_decorator(user_passes_test(lambda u: u.has_perm('Management.view_expenseitem'), login_url='/auth/login/'))
    def get(self,request):
        print(request.data)
        reponse_data = {"type" : "whatsapp", "data" : [ {"id" : 2, "wid" : "+201018223004", "phone" : "+201155186508", "message" : "Hello World!"}]}
        reponse_list = [{"type" : "whatsapp", # type of payload: received whatsapp chat
                "data" : [
                "id" : 2, # unique id from the system
                "wid" : "+639760713666", # whatsapp account phone number
                "phone" : "+639760666713", # sender phone number
                "message" : "Hello World!", # message
                #"attachment" => "http:#imageurl.com/image.jpg", # attachment url
                #"timestamp" => 1645684231 # receive timestamp
            ]},
        ]
        return Response(reponse_data,status=status.HTTP_200_OK)
    
    def post(self,request):
        print(request.data)
        reponse_data = {"type" : "whatsapp", "data" : [ {"id" : 2, "wid" : "+201018223004", "phone" : "+201155186508", "message" : "Hello World!"}]}
        reponse_list = [{"type" : "whatsapp", # type of payload: received whatsapp chat
                "data" : [
                "id" : 2, # unique id from the system
                "wid" : "+639760713666", # whatsapp account phone number
                "phone" : "+639760666713", # sender phone number
                "message" : "Hello World!", # message
                #"attachment" => "http:#imageurl.com/image.jpg", # attachment url
                #"timestamp" => 1645684231 # receive timestamp
            ]},
        ]
        return Response(reponse_data,status=status.HTTP_200_OK)
#http://vmi873706.contaboserver.net:7777/br_rpt/test_whatsapp_api_i/
class TestWhatsappApiInfobip(APIView):
    #@method_decorator(user_passes_test(lambda u: u.has_perm('Management.view_expenseitem'), login_url='/auth/login/'))
    def get(self,request):
        print(request.data)
        reponse_data = {"type" : "whatsapp", "data" : [ {"id" : 2, "wid" : "+201018223004", "phone" : "+201155186508", "message" : "Hello World!"}]}
        reponse_list = [{"type" : "whatsapp", # type of payload: received whatsapp chat
                "data" : [
                "id" : 2, # unique id from the system
                "wid" : "+639760713666", # whatsapp account phone number
                "phone" : "+639760666713", # sender phone number
                "message" : "Hello World!", # message
                #"attachment" => "http:#imageurl.com/image.jpg", # attachment url
                #"timestamp" => 1645684231 # receive timestamp
            ]},
        ]
        return Response(reponse_data,status=status.HTTP_200_OK)
    
    def post(self,request):
        print(request.data)
        reponse_data = {"type" : "whatsapp", "data" :{"id" : 2, "wid" : "+201018223004", "phone" : "+201155186508", "message" : "Hello World!","pt_name":"برسوم سمير" ,"ms_name":"كشف" ,"visit_order":"25","waiting_count":"6"}}
        reponse_list = [{"type" : "whatsapp", # type of payload: received whatsapp chat
                "data" : [
                "id" : 2, # unique id from the system
                "wid" : "+639760713666", # whatsapp account phone number
                "phone" : "+639760666713", # sender phone number
                "message" : "Hello World!", # message
                #"attachment" => "http:#imageurl.com/image.jpg", # attachment url
                #"timestamp" => 1645684231 # receive timestamp
            ]},
        ]
        return Response(reponse_data,status=status.HTTP_200_OK)

'''
class BranchOutcomeDetailsApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_BranchOutcomeDetails'), login_url='/auth/login/'))
    def get(self,request):
        branches = Branch.objects.filter(is_active=True).values('id','name')
        all_expense_cats =ExpenseCategory.objects.all()
        return render(request,"Clinic/brreportApp/branch_outcome_details.html",{
            "branches":branches,"expense_cats":all_expense_cats,'activePage':'BranchOutcomeDetails'})
    
    def post(self,request):
        response_data={}
        data=request.data
        data._mutable=True
        CurrentPeriodstartDateTime,CurrentPeriodendDateTime = getStartEndDateTime(data["startDate"],data["endDate"])
        branch_id = data["branch_id"]
        expense_category_id = data["expense_category_id"]
        q_list = [Q(created_on__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]),Q(branch_id=branch_id)]
        if expense_category_id != "0":q_list.append(Q(item__category_id=expense_category_id))
        expenses = Expense.objects.filter(reduce(operator.and_, q_list))
        items_sum_data = expenses.values("item__name").annotate(item_price_sum = Sum('price'))
        response_data['items_sum_data'] = {"names": list(map(lambda item: item['item__name'], items_sum_data)),"values":list(map(lambda item: item['item_price_sum'], items_sum_data))}
        items_count_data = expenses.values("item__name").annotate(item_count = Count('id'))
        response_data['items_count_data'] = {"names": list(map(lambda item: item['item__name'], items_count_data)),"values":list(map(lambda item: item['item_count'], items_count_data))}
        expenses_srl=ExpenseCustomSerializer(expenses.order_by("item","-created_on"),many=True)
        response_data["expenses_detail"]=expenses_srl.data
        #items_count_data_names = []
        #items_count_data_values = []
        # all_items = expenses.values_list('item__name', flat=True)
        # # Count the occurrences of each item using Counter
        # item_counts = Counter(all_items)
        # # Print or use the counts
        # for item_name, count in item_counts.items(): 
        #     items_count_data_names.append(item_name)
        #     items_count_data_values.append(count)
        # for item_name, count in item_counts.items(): 
        #     items_count_data_names.append(item_name)
        #     items_count_data_values.append(count)
        #response_data['items_chart_data'] = {"names": list(map(lambda item: item['item__name'], items_data)),"values":list(map(lambda item: item['item_count'], items_data))}
        return Response(response_data,status=status.HTTP_200_OK)
    
from brvisitApp.models import PvPayment
from itertools import chain # Join Multiple Querysets From Different Base Models Django
from brmainApp.utils import *
import json

class BranchIncomeDetailsApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_BranchIncomeDetails'), login_url='/auth/login/'))
    def get(self,request):
        branches = Branch.objects.filter(is_active=True).values('id','name')
        return render(request,"Clinic/brreportApp/branch_income_details.html",{
            "branches":branches,'activePage':'BranchIncomeDetails'})
    
    def post(self,request):
        data=request.data
        data._mutable=True
        CurrentPeriodstartDateTime,CurrentPeriodendDateTime = getStartEndDateTime(data["startDate"],data["endDate"])
        branch_id = data["branch_id"]
        q_list__pv = [Q(created_at__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]),Q(amount_paid__gt=0),Q(visit_branch_id=branch_id)]
        q_list__pv_payments = [Q(created_at__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]),Q(pv__visit_branch_id=branch_id)]
        q_list__ps = [Q(created_at__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]),Q(amount_paid__gt=0),Q(session__branch_id=branch_id)]
        q_list__ps_payments = [Q(created_at__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]),Q(pt_ssn__session__branch_id=branch_id)]

        # get visits income 
        pv_data = PatientVisit.objects.filter(reduce(operator.and_, q_list__pv)).values("id","patient__fullname","medicalservice__name","amount_paid","created_at","created_by__name")
        for item in pv_data: 
            #item["patient__fullname"] = item.get('patient__fullname')
            item["payment_category"] = item.get('medicalservice__name')
            del item["medicalservice__name"]# remove old key and its value 
        # get additional PvPayment 
        pv_payments_data = PvPayment.objects.filter(reduce(operator.and_, q_list__pv_payments)).values("id","pv__patient__fullname","amount_paid","created_at","created_by__name")
        for item in pv_payments_data: 
            item["patient__fullname"] = item.get('pv__patient__fullname')
            item["payment_category"] = "استكمال مدفوعات زيارة"
            del item["pv__patient__fullname"]# remove old key and its value
        # get new sessions income 
        ps_data = PatientSession.objects.filter(reduce(operator.and_, q_list__ps)).values("id","patient__fullname","session__name","amount_paid","created_at","created_by__name")
        for item in ps_data: 
            #item["patient__fullname"] = item.get('pv__patient__fullname')
            item["payment_category"] = item.get('session__name')
            del item["session__name"]# remove old key and its value
        # get sessions payments 
        ps_payments_data = PtSsnPayment.objects.filter(reduce(operator.and_, q_list__ps_payments)).values("id","pt_ssn__patient__fullname","amount_paid","created_at","created_by__name")
        for item in ps_payments_data: 
            item["patient__fullname"] = item.get('pt_ssn__patient__fullname')
            item["payment_category"] = "استكمال مدفوعات جلسة"
            del item["pt_ssn__patient__fullname"]# remove old key and its value

        invoices_relatedPayments = chain(pv_data, pv_payments_data,ps_data,ps_payments_data) #Join Multiple Querysets From Different Base Models Django
        data_sorted = sorted(invoices_relatedPayments, key=lambda x: x['created_at'], reverse=True)# sort list of dict by value of Key (created_at)
        data_sorted_json = [json.dumps(i,default=jsonSerializerIfDateTime) for i in data_sorted] 
        return Response({"income_details_data":data_sorted_json},status=status.HTTP_200_OK)
    
from collections import defaultdict
class BranchIncomeOutcomeApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_BranchIncomeOutcome'), login_url='/auth/login/'))
    def get(self,request):
        branches = Branch.objects.filter(is_active=True).values('id','name')
        return render(request,"Clinic/brreportApp/branch_incomes_outcomes.html",{
            "branches":branches,'activePage':'BranchIncomeOutcome'})
    
    def post(self,request):
        response_data={}
        data=request.data
        data._mutable=True
        CurrentPeriodstartDateTime,CurrentPeriodendDateTime = getStartEndDateTime(data["startDate"],data["endDate"])
        #CurrentPeriodstartDateTime,CurrentPeriodendDateTime = getStartEndDateTime(data["startDate"][0],data["endDate"][0])
        branch_id = data["branch_id"]
        # visits - sessions items - expenses 
        response_data={}
        q_list_visits = [Q(visit_day__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]),Q(visit_branch_id=branch_id)]
        visits = PatientVisit.objects.filter(reduce(operator.and_, q_list_visits)).values('medicalservice_id','medicalservice__name','amount_paid','related_payment','created_by')
        medicalservices_data = visits.values('medicalservice__name').annotate(ms_count=Count('medicalservice__name'),total_amount_paid=Sum('amount_paid'),total_related_payment=Sum('related_payment'))
        #المفروض نعمل الريلاتيد بايمينت بنفس الطريقة اللى تحت عشان يبقي الرقم دقيق بس هنسيبها مؤقتا على اعتبار ان المريض بيدفع مبلغ الكشف كامل فى الاول 
        medicalservices_data_list=[]
        for ms_data in medicalservices_data : medicalservices_data_list.append({"ms_name":ms_data["medicalservice__name"],"ms_count":ms_data["ms_count"],"amount_paid":ms_data["total_amount_paid"],"related_payment":ms_data["total_related_payment"]})
        response_data["medicalservices_data"]=medicalservices_data_list
        q_list_pt_sessions = [Q(created_at__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]),Q(patient__branch_id=branch_id)]
        #q_list_pt_sessions = [Q(created_at__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]),Q(patient__branch_id=branch_id[0])]
        #pt_sessions = PatientSession.objects.filter(reduce(operator.and_, q_list_pt_sessions)).values("id","session_id","initial_payment","patient__fullname","session__name")
        pt_sessions = PatientSession.objects.filter(reduce(operator.and_, q_list_pt_sessions)).values("session_id","initial_payment","created_by")
        pt_sessions_data = pt_sessions.values("session_id").annotate(total_initial_payment=Sum('initial_payment'))

        q_list_session_items = [Q(session_day__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]),Q(session__patient__branch_id=branch_id)]
        session_items = PatientSessionItem.objects.filter(reduce(operator.and_, q_list_session_items)).values('id',"session__session_id","session_status__code")
        session_items_data = session_items.values("session__session_id").annotate(total_count=Count('id'))
        session_items_attend = session_items.filter(session_status_id="2AT").values("session__session_id").annotate(attend_count=Count('id'))
        q_list_related_payments = [Q(created_at__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]),Q(pt_ssn__patient__branch_id=branch_id)]
        related_payments = PtSsnPayment.objects.filter(reduce(operator.and_, q_list_related_payments)).values("pt_ssn__session_id","amount_paid","created_by")
        #q_list_related_payments = [Q(created_at__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]),Q(pt_ssn__patient__branch_id=branch_id[0])]
        #related_payments = PtSsnPayment.objects.filter(reduce(operator.and_, q_list_related_payments)).values("pt_ssn__session_id","amount_paid","pt_ssn__session__name","pt_ssn__patient__fullname","id")
        related_payments_data = related_payments.values("pt_ssn__session_id").annotate(total_paid=Sum('amount_paid'))
        all_sessions_data_list=[]
        all_sessions = Session.objects.filter(branch_id=branch_id).values("id","name")
        for session in all_sessions :
            session_id = session["id"]
            session_obj ={"id":session["id"],"name":session["name"]}
            session_total_assigned_data = session_items_data.filter(session__session_id=session_id) 
            session_obj["total_assigned"] = session_total_assigned_data[0]["total_count"] if session_total_assigned_data else 0
            session_total_attend_data = session_items_attend.filter(session__session_id=session_id) 
            session_obj["total_attend"] = session_total_attend_data[0]["attend_count"] if session_total_attend_data else 0
            session_total_initial_payment = pt_sessions_data.filter(session_id=session_id) 
            session_obj["total_initial_payments"] = session_total_initial_payment[0]["total_initial_payment"] if session_total_initial_payment else 0
            session_total_related_payments = related_payments_data.filter(pt_ssn__session_id=session_id) 
            session_obj["total_related_payments"] = session_total_related_payments[0]["total_paid"] if session_total_related_payments else 0
            all_sessions_data_list.append(session_obj)

        response_data["sessions_data"]=all_sessions_data_list
        q_list_expenses = [Q(created_on__range=[CurrentPeriodstartDateTime,CurrentPeriodendDateTime]),Q(branch_id=branch_id)]
        expenses = Expense.objects.filter(reduce(operator.and_, q_list_expenses))
        expenses_data = expenses.values("item_id").annotate(total_count=Count('id'),item_price_sum = Sum('price'))
        #expenses_users_data = expenses.values("created_by").annotate(item_price_sum = Sum('price'))
        expenses_data_list=[]
        all_expense_items = ExpenseItem.objects.filter(branch_id=branch_id).values("id","name","category__name")
        for item in all_expense_items :
            item_id = item["id"]
            item_obj ={"id":item["id"],"name":item["name"],"category_name":item["category__name"]}
            item_expenses = expenses_data.filter(item_id=item_id)
            item_obj["total_count"] = item_expenses[0]["total_count"] if item_expenses else 0
            item_obj["total_price"] = item_expenses[0]["item_price_sum"] if item_expenses else 0
            expenses_data_list.append(item_obj)
        response_data["expenses_data"]=expenses_data_list
        amount_paid_visits = visits.values("created_by").annotate(amount_paid_sum = Sum('amount_paid'))
        related_payment_visits = visits.values("created_by").annotate(amount_paid_sum = Sum('related_payment'))
        initial_payment_ssn = pt_sessions.values("created_by").annotate(amount_paid_sum = Sum('initial_payment'))
        related_payments_ssn = related_payments.values("created_by").annotate(amount_paid_sum = Sum('amount_paid'))
        # Initialize a defaultdict to store combined results
        combined_dict = defaultdict(int)
        # Combine all QuerySets
        for qs in [amount_paid_visits, related_payment_visits, initial_payment_ssn,related_payments_ssn]:
            for record in qs:
                combined_dict[record['created_by']] += record['amount_paid_sum']
        #import pdb
        #pdb.set_trace()
        expenses_users_data = expenses.values("created_by").annotate(item_price_sum = Sum('price'))
        expenses_users_ids = [item['created_by'] for item in expenses_users_data]
        for record in expenses_users_data:combined_dict[record['created_by']] -= record['item_price_sum']
        combined_dict = dict(combined_dict)
        users_ids = list(combined_dict.keys())
        users_ids.extend(expenses_users_ids)
        rcp_user_data = User.objects.filter(id__in=users_ids).values('id','name')
        # Step 1: Create a mapping of id to name from QuerySet y
        id_to_name = {item['id']: item['name'] for item in rcp_user_data}
        # Step 2: Replace the keys in dictionary x with names from the id_to_name mapping
        #output : {'مستخدم جديد': 7806.0, 'احمد موسي': 6050.0}
        total_income_outcome_data = {id_to_name[key]: value for key, value in combined_dict.items()}
        total_data_chart = {"names": list(total_income_outcome_data.keys()),"values":list(total_income_outcome_data.values())}
        expenses_user_names = [ id_to_name[item['created_by']] for item in expenses_users_data]
        expenses_data_chart = {"names": expenses_user_names,"values":list(map(lambda item: item['item_price_sum'], expenses_users_data))}
        response_data ["charts_data"] = {"total_data_chart":total_data_chart,"expenses_data_chart":expenses_data_chart}
        return Response(response_data,status=status.HTTP_200_OK)